# From Imports
from os import path as osPath, name as osName
from pathlib import Path as libPath
from typing import List, Callable

# Local Imports
from nanite import nanite

isf: Callable[..., bool] = osPath.isfile
fullpath: Callable[..., str] = nanite().fullpath


class ignores:
    def __init__(
        self, globList: List[str] = [], regexList: List[str] = []
    ):
        """
            Variables "globList" and "regexList" take an iterable
            (list, tuple, generator, etc.) of exclusion paths in
            the "glob" and "regex" formats respectively.

            Elements of the glob list containing environment variables,
            or a tilde ( ~ ) will be expanded.

            In addition, all paths with environment variables or tildes
            will be expanded as well.
        """
        self.globList: List[str] = globList
        self.regexList: List[str] = regexList

    # Creates a ".hgignore" file
    def hg(self, source: str) -> None:
        """
            Create and populate the "mercurial" ignore file
        """
        ignoreFile: str = fullpath(source, ".hgignore")
        if isf(ignoreFile):
            print("Using existing file!")
        else:
            # Use the context manager from the O'Reilly article here...?
            with open(ignoreFile, "w") as ignore:
                if self.globList:
                    ignore.write("syntax: glob\n\n")
                    for excl in self.globList:
                        ignore.write(f"{excl}\n")
                    ignore.write("\n")
                if self.regexList:
                    ignore.write("syntax: regexp\n\n")
                    for excl in self.regexList:
                        ignore.write(f"{excl}\n")
                    ignore.write("\n")

    # Creates a ".gitignore" file
    def git(self, source: str, fmList: List[str]) -> None:
        """
            Create and populate the "git" ignore file
        """
        ignoreFile: str = fullpath(source, ".gitignore")
        if isf(ignoreFile):
            print("Using existing file!")
        else:
            if fmList:
                # Use the context manager from the O'Reilly article here...?
                with open(ignoreFile, "w") as ignore:
                    for excl in fmList:
                        ignore.write(f"{excl}\n")

    # Creates an fd ignore file
    def fd(self, ignoreFile: str) -> None:
        """
            Create and populate the "fd" ignore file
        """
        with open(fullpath(ignoreFile), "w") as ignore:
            if self.globList:
                for excl in self.globList:
                    ignore.write(f"{excl}\n")

    def glob(self) -> str:
        """
            Create the necessary bracketed glob pattern for the glob module
        """
        return "[!" + "][!".join(self.globList) + "]"

    # Creates a borg ignore file
    def borg(
        self,
        ignoreFile: str,
        fmList: List[str] = None,
        ppList: List[str] = None,
        pfList: List[str] = None,
    ) -> None:
        """
            Create and populate the "borg" ignore file
        """
        with open(fullpath(ignoreFile), "w") as ignore:
            if self.globList:
                for excl in self.globList:
                    ignore.write(f"sh:{excl}\n")
            if self.regexList:
                for excl in self.regexList:
                    ignore.write(f"re:{excl}\n")
            if fmList:
                for excl in fmList:
                    ignore.write(f"fm:{excl}\n")
            if ppList:
                for excl in ppList:
                    ignore.write(f"pp:{excl}\n")
            if pfList:
                for excl in pfList:
                    ignore.write(f"pf:{excl}\n")
